@@ -58,10 +58,10 @@ def translate_labellings(trans_from_labels,trans_to_labels,subsample_size='none'
     return np.array([trans_dict[l] for l in trans_from_labels])
 
 def get_fanout_trans_dict(trans_from_labels,trans_to_labels,subsample_size):
-    unique_trans_from_labels = unique_labels(trans_from_labels)
-    unique_trans_to_labels = unique_labels(trans_to_labels)
+    unique_trans_from_labels = [l for l in unique_labels(trans_from_labels) if l != -1]
+    unique_trans_to_labels = [l for l in unique_labels(trans_to_labels) if l != -1]
     if subsample_size == 'none':
-        cost_matrix = np.array([[label_assignment_cost(trans_from_labels,trans_to_labels,l1,l2) for l2 in unique_trans_to_labels if l2 != -1] for l1 in unique_trans_from_labels if l1 != -1])
+        cost_matrix = np.array([[label_assignment_cost(trans_from_labels,trans_to_labels,l1,l2) for l2 in unique_trans_to_labels] for l1 in unique_trans_from_labels])
     else:
         num_trys = 0
         while True:
@@ -70,8 +70,8 @@ def get_fanout_trans_dict(trans_from_labels,trans_to_labels,subsample_size):
             sample_indices = np.random.choice(range(trans_from_labels.shape[0]),subsample_size,replace=False)
             trans_from_labels_subsample = trans_from_labels[sample_indices]
             trans_to_labels_subsample = trans_to_labels[sample_indices]
-            if unique_labels(trans_from_labels_subsample) == unique_trans_from_labels and unique_labels(trans_to_labels_subsample) == unique_trans_to_labels: break
-        cost_matrix = np.array([[label_assignment_cost(trans_from_labels_subsample,trans_to_labels_subsample,l1,l2) for l2 in unique_trans_from_labels if l2 != -1] for l1 in unique_trans_from_labels if l1 != -1])
+            if unique_labels(trans_from_labels_subsample) == set(unique_trans_from_labels) and unique_labels(trans_to_labels_subsample) == set(unique_trans_to_labels): break
+        cost_matrix = np.array([[label_assignment_cost(trans_from_labels_subsample,trans_to_labels_subsample,l1,l2) for l2 in unique_trans_from_labels] for l1 in unique_trans_from_labels])
     row_ind, col_ind = linear_sum_assignment(cost_matrix)
     if len(col_ind) != len(set(trans_from_labels[trans_from_labels != -1])):
         raise TranslationError(f"then translation cost matrix is the wrong size for some reason")
@@ -80,10 +80,10 @@ def get_fanout_trans_dict(trans_from_labels,trans_to_labels,subsample_size):
     return trans_dict
 
 def get_fanin_trans_dict(trans_from_labels,trans_to_labels,subsample_size):
-    unique_trans_from_labels = unique_labels(trans_from_labels)
-    unique_trans_to_labels = unique_labels(trans_to_labels)
+    unique_trans_from_labels = [l for l in unique_labels(trans_from_labels) if l != -1]
+    unique_trans_to_labels = [l for l in unique_labels(trans_to_labels) if l != -1]
     if subsample_size == 'none':
-        cost_matrix = np.array([[label_assignment_cost(trans_to_labels,trans_from_labels,l1,l2) for l2 in unique_trans_from_labels if l2 != -1] for l1 in unique_trans_to_labels if l1 != -1])
+        cost_matrix = np.array([[label_assignment_cost(trans_to_labels,trans_from_labels,l1,l2) for l2 in unique_trans_from_labels] for l1 in unique_trans_to_labels])
     else:
         while True: # Keep trying random indices unitl you reach one that contains all labels
             sample_indices = np.random.choice(range(trans_from_labels.shape[0]),subsample_size,replace=False)
@@ -93,18 +93,21 @@ def get_fanin_trans_dict(trans_from_labels,trans_to_labels,subsample_size):
         sample_indices = np.random.choice(range(trans_from_labels.shape[0]),subsample_size,replace=False)
         trans_from_labels_subsample = trans_from_labels[sample_indices]
         trans_to_labels_subsample = trans_to_labels[sample_indices]
-        cost_matrix = np.array([[label_assignment_cost(trans_to_labels_subsample,trans_from_labels_subsample,l1,l2) for l2 in unique_trans_from_labels if l2 != -1] for l1 in unique_trans_to_labels if l1 != -1])
+        cost_matrix = np.array([[label_assignment_cost(trans_to_labels_subsample,trans_from_labels_subsample,l1,l2) for l2 in unique_trans_from_labels] for l1 in unique_trans_to_labels])
     row_ind, col_ind = linear_sum_assignment(cost_matrix)
     if len(col_ind) != get_num_labels(trans_to_labels):
         raise TranslationError(f"then translation cost matrix is the wrong size for some reason")
     cl = col_ind.tolist()
     trans_dict = {f:cl.index(f) for f in cl}
+    num_trys = 0
     while True:
+        num_trys += 1
+        if num_trys == 5: set_trace()
         untranslated = [i for i in unique_trans_from_labels if i not in trans_dict.keys()]
         if len(untranslated) == 0: break
         unique_untranslated = unique_labels(untranslated)
         # Now assign the additional, unassigned items
-        cost_matrix2 = np.array([[label_assignment_cost(trans_from_labels,trans_to_labels,l1,l2) for l2 in unique_trans_to_labels if l2 != -1] for l1 in unique_untranslated if l1 != -1])
+        cost_matrix2 = np.array([[label_assignment_cost(trans_from_labels,trans_to_labels,l1,l2) for l2 in unique_trans_to_labels] for l1 in unique_untranslated])
         row_ind2, col_ind2 = linear_sum_assignment(cost_matrix2)
         for u,t in zip(untranslated,col_ind2): trans_dict[u]=t
     trans_dict[-1] = -1
